# HG changeset patch
# User Maikel de Vries
# Date 1323615559 -3600
# Node ID becfb84ca2e774e3cf00d215172fbe64515e77ec
# Parent  da984ec0ed73d3dc1eee4d5f07feaf711c415ce4
Implemented drag and drop images for meshes

diff -r da984ec0ed73 -r becfb84ca2e7 src/gui/C4MouseControl.cpp
--- a/src/gui/C4MouseControl.cpp	Sun Dec 11 15:50:51 2011 +0100
+++ b/src/gui/C4MouseControl.cpp	Sun Dec 11 15:59:19 2011 +0100
@@ -140,6 +140,8 @@
 	DownRegion.Default();
 	DragImage.Default();
 	DragImagePhase=0;
+	DragImageID=C4ID::None;
+	DragImageObject=NULL;
 	fMouseOwned = true; // default mouse owned
 	fctViewport.Default();
 }
@@ -432,7 +434,7 @@
 				if (!IsPassive())
 					fAddMark=true;
 		// Drag image
-		if (DragImage.Surface)
+		if (DragImage.Surface || DragImageObject)
 		{
 			// zoom mode: Drag in GUI or Game depending on source object
 			bool fIsGameZoom = true;
@@ -455,6 +457,16 @@
 			}
 			// draw in special modulation mode
 			pDraw->SetBlitMode(C4GFXBLIT_MOD2);
+			float ImageWdt = DragImage.Wdt;
+			if (DragImageObject)
+			{
+				C4DefGraphics* pGfx = DragImageObject->GetGraphics();
+				if (pGfx->Type == C4DefGraphics::TYPE_Bitmap)
+					; // TODO
+				else
+					pGfx->
+					ImageWdt = pGfx->Mesh->GetBoundingBox().x2 - pGfx->Mesh->GetBoundingBox().x1; // TODO: 
+			}
 			// draw DragImage in red or green, according to the phase to be used
 			iOffsetX=int(ZoomDraw*DragImage.Wdt/2);
 			if (Drag == C4MC_Drag_Construct)
@@ -462,10 +474,18 @@
 			else
 				iOffsetY=int(ZoomDraw*DragImage.Hgt/2);
 			pDraw->ActivateBlitModulation((Drag == C4MC_Drag_Script) ? 0x7fffffff : (DragImagePhase ? 0x8f7f0000 : 0x1f007f00));
-			pDraw->Blit(DragImage.Surface,
+
+			if (DragImageObject)
+			{
+				DragImageObject->Draw(cgo,-1,C4Object::ODM_Normal,50,50);
+			}
+			else
+			{
+				pDraw->Blit(DragImage.Surface,
 			              float(DragImage.X), float(DragImage.Y), float(DragImage.Wdt), float(DragImage.Hgt),
 			              cgo.Surface,
 			              XDraw + cgo.X - iOffsetX, YDraw + cgo.Y - iOffsetY, float(DragImage.Wdt)*ZoomDraw, float(DragImage.Hgt)*ZoomDraw,true);
+			}
 			// reset color
 			pDraw->DeactivateBlitModulation();
 			pDraw->SetBlitMode(0);
@@ -796,6 +816,8 @@
 			{
 				Drag=C4MC_Drag_Script;
 				CreateDragImage(drag_image_id, drag_image_obj, true);
+				if(drag_image_obj) DragImageObject = drag_image_obj;
+				else DragImageID = drag_image_id;
 				DragObject = TargetObject;
 				DragImagePhase=0;
 			}
@@ -1167,7 +1189,8 @@
 {
 	Drag=C4MC_Drag_Construct;
 	DragID=id;
-	CreateDragImage(DragID,NULL,false);
+	//CreateDragImage(DragID,NULL,false);
+	DragImageID = id;
 	Selection.Clear();
 }
 
diff -r da984ec0ed73 -r becfb84ca2e7 src/gui/C4MouseControl.h
--- a/src/gui/C4MouseControl.h	Sun Dec 11 15:50:51 2011 +0100
+++ b/src/gui/C4MouseControl.h	Sun Dec 11 15:59:19 2011 +0100
@@ -86,6 +86,8 @@
 	C4ID DragID;
 	C4ObjectList Selection;
 	C4Facet DragImage;
+	C4ID DragImageID;
+	C4Object* DragImageObject;
 	// Target object
 	C4Object *TargetObject; // valid during Move()
 	C4Object *DownTarget;
